"""
Internal management commands of the **shelter** library.
"""


SHELTER_MANAGEMENT_COMMANDS = (
    'shelter.commands.devserver.DevServer',
    'shelter.commands.runserver.RunServer',
    'shelter.commands.shell.Shell',
    'shelter.commands.showconfig.ShowConfig',
    'shelter.commands.startproject.StartProject',
)
"""
Library internal management commands.
"""
